/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

using namespace vcg;    

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  srand(time(0)); // initialize the random number generator

  // let delete a random number of faces
  for(int i=0;i<m.FN()/20;++i)
    {
      int face_to_delete = rand()%m.face.size();
      if(!m.face[face_to_delete].IsD())
        tri::Allocator<MyMesh>::DeleteFace(m,m.face[face_to_delete]);
    }

  tri::UpdateTopology<MyMesh>::FaceFace(m);
  // to avoid checking deleted face let's compact the mesh
  tri::Allocator<MyMesh>::CompactFaceVector(m);

  // Try to identify non manifold vertices and mark them using only the face face topology

  tri::UpdateFlags<MyMesh>::VertexClearV(m);
  tri::UpdateFlags<MyMesh>::FaceClearV(m);
   
  // Loop over all the faces and 
  // for each vertex of each face loop over that vertex by using the FF topology (Doing flipF and flipE)
  // if the face is not visited yet mark it as visited and mark the vertex as visited
  // Main idea: if the vertex is visited twice then it is non manifold
  // so if we find a vertex that is visited from a face that has not been visited yet we can mark it as non manifold
  // we assume compact face vector

  // we need to allocate three new user bitflags for each face to mark the visited faces 
  // according the vertex we are considering
  int bf[3] = { MyFace::NewBitFlag(), MyFace::NewBitFlag(), MyFace::NewBitFlag() };
  

  
  // save the mesh with the border faces colored in red
  tri::io::ExporterOFF<MyMesh>::Save(m,"border.off",tri::io::Mask::IOM_FACECOLOR);
  return 0;
}
